"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const react_native_reanimated_1 = __importStar(require("react-native-reanimated"));
const react_native_gesture_handler_1 = require("react-native-gesture-handler");
const image_editor_1 = __importDefault(require("@react-native-community/image-editor"));
const react_native_image_size_1 = __importDefault(require("react-native-image-size"));
const theme_1 = require("../Styles/theme");
const react_2 = require("react");
const react_3 = require("react");
const Icon_1 = __importDefault(require("../Icon"));
const Spin_1 = __importDefault(require("../Spin"));
const Toast_1 = __importDefault(require("../Toast"));
const { width: WINDOW_WIDTH, height: WINDOW_HEIGHT } = react_native_1.Dimensions.get('window');
const ImageCropper = props => {
    const { uri, visible, destSize = { width: 500, height: 500 }, cancelText = '取消', okText = '确定', onConfirm, onClose, } = props;
    const [imageInfo, setImageInfo] = react_3.useState({
        originWidth: 0,
        originHeight: 0,
        width: 0,
        height: 0,
        x: 0,
        y: 0,
    });
    const [rectInfo, setReactInfo] = react_3.useState({ width: 0, height: 0, x: 0, y: 0 });
    const [loading, setLoading] = react_3.useState(false);
    const scale = react_native_reanimated_1.useSharedValue(1);
    const savedScale = react_native_reanimated_1.useSharedValue(1);
    const positionX = react_native_reanimated_1.useSharedValue(0);
    const positionY = react_native_reanimated_1.useSharedValue(0);
    const savedPositionX = react_native_reanimated_1.useSharedValue(0);
    const savedPositionY = react_native_reanimated_1.useSharedValue(0);
    react_2.useEffect(() => {
        (async () => {
            if (visible && uri) {
                setLoading(true);
                let { width, height, rotation } = await react_native_image_size_1.default.getSize(uri);
                // 安卓上，使用ios的图片，可能会有图片角度问题
                if (rotation === 90 || rotation === 270) {
                    [width, height] = [height, width];
                }
                // 宽取 两个中最小的一个，最小不小于0.8屏幕宽
                const max = Math.max(width, WINDOW_WIDTH);
                let w = Math.min(WINDOW_WIDTH * 0.8, Math.min(width, WINDOW_WIDTH));
                const rate = w / max;
                let h = height * rate;
                if (h > WINDOW_HEIGHT) {
                    w = (w * WINDOW_HEIGHT * 0.8) / h;
                    h = WINDOW_HEIGHT * 0.8;
                }
                const x = (WINDOW_WIDTH - w) / 2;
                const y = (WINDOW_HEIGHT - h) / 2;
                setImageInfo({
                    originWidth: width,
                    originHeight: height,
                    width: w,
                    height: h,
                    x,
                    y,
                });
                const size = Math.min(w, h);
                setReactInfo({
                    width: size,
                    height: size,
                    x: (WINDOW_WIDTH - size) / 2,
                    y: (WINDOW_HEIGHT - size) / 2,
                });
                setLoading(false);
            }
        })();
    }, [uri, visible]);
    const pinchGesture = react_native_gesture_handler_1.Gesture.Pinch()
        .onUpdate(e => {
        scale.value = savedScale.value * e.scale;
    })
        .onEnd(() => {
        // 不能比原图小
        if (scale.value <= 1) {
            scale.value = 1;
            savedScale.value = 1;
        }
        else {
            savedScale.value = scale.value;
        }
    });
    const panGesture = react_native_gesture_handler_1.Gesture.Pan()
        .maxPointers(1)
        .onUpdate(e => {
        positionX.value = savedPositionX.value + e.translationX;
        positionY.value = savedPositionY.value + e.translationY;
    })
        .onEnd(() => {
        // 缩放后，每边可以用于移动的宽度
        const scaledWidth = (imageInfo.width * (scale.value - 1)) / 2;
        // 相对于缩放的实际移动值
        const translateX = positionX.value * scale.value;
        // 图片与裁剪框之间的偏移量
        const offsetX = rectInfo.x - imageInfo.x;
        // 计算后，实际每边可以用于移动的宽度
        const targetX = scaledWidth + offsetX;
        let destX = positionX.value;
        // 左边移动进了裁剪框，translateX正数是右移
        if (translateX > targetX) {
            destX = targetX / scale.value;
        }
        else if (-translateX > targetX) {
            // 右边移进了裁剪框
            destX = -targetX / scale.value;
        }
        positionX.value = destX;
        savedPositionX.value = destX;
        const scaledHeight = (imageInfo.height * (scale.value - 1)) / 2;
        const translateY = positionY.value * scale.value;
        const offsetY = rectInfo.y - imageInfo.y;
        const targetY = scaledHeight + offsetY;
        let destY = positionY.value;
        // 左边移动进了裁剪框
        if (translateY > targetY) {
            destY = targetY / scale.value;
        }
        else if (-translateY > targetY) {
            // 右边移进了裁剪框
            destY = -targetY / scale.value;
        }
        positionY.value = destY;
        savedPositionY.value = destY;
    });
    const pinchAnimStyle = react_native_reanimated_1.useAnimatedStyle(() => ({
        transform: [{ scale: scale.value }, { translateX: positionX.value }, { translateY: positionY.value }],
    }));
    const onCrop = async () => {
        const scaledWidth = imageInfo.width * (scale.value - 1);
        const x = imageInfo.x - scaledWidth / 2 + positionX.value;
        const scaledHeight = imageInfo.height * (scale.value - 1);
        const y = imageInfo.y - scaledHeight / 2 + positionY.value;
        const destX = (rectInfo.x - x) / scale.value;
        const destY = (rectInfo.y - y) / scale.value;
        setLoading(true);
        const url = await image_editor_1.default.cropImage(uri, {
            offset: { x: destX, y: destY },
            size: {
                width: (rectInfo.width * imageInfo.originWidth) / imageInfo.width / scale.value,
                height: (rectInfo.height * imageInfo.originHeight) / imageInfo.height / scale.value,
            },
            displaySize: destSize,
        })
            .catch(error => {
            Toast_1.default.info('图片裁剪失败');
            console.error('图片裁剪失败: ', error);
        })
            .finally(() => setLoading(false));
        if (url) {
            onConfirm(url);
            onClose();
        }
    };
    const renderContent = react_native_gesture_handler_1.gestureHandlerRootHOC(() => (react_1.default.createElement(react_native_1.View, { style: styles.container },
        loading && (react_1.default.createElement(react_native_1.View, { style: styles.loading },
            react_1.default.createElement(Spin_1.default, { size: 40, color: theme_1.Theme.colorTextBaseInverse }))),
        react_1.default.createElement(react_native_reanimated_1.default.View, { style: [styles.imageWrap, { left: imageInfo.x, top: imageInfo.y }, pinchAnimStyle] },
            react_1.default.createElement(react_native_1.Image, { source: { uri }, style: { width: imageInfo.width, height: imageInfo.height } })),
        react_1.default.createElement(react_native_gesture_handler_1.GestureDetector, { gesture: pinchGesture },
            react_1.default.createElement(react_native_gesture_handler_1.GestureDetector, { gesture: panGesture },
                react_1.default.createElement(react_native_1.View, { style: styles.gesture },
                    react_1.default.createElement(react_native_1.View, { style: [
                            styles.rect,
                            { width: rectInfo.width, height: rectInfo.height, left: rectInfo.x, top: rectInfo.y },
                        ] }),
                    react_1.default.createElement(react_native_1.View, { style: styles.footer },
                        react_1.default.createElement(react_native_1.Pressable, { hitSlop: 10, onPress: onClose },
                            react_1.default.createElement(react_native_1.Text, { style: styles.text }, cancelText)),
                        react_1.default.createElement(react_native_1.Pressable, { hitSlop: 10 },
                            react_1.default.createElement(Icon_1.default, { name: "undo", color: theme_1.Theme.colorTextBaseInverse, size: 24 })),
                        react_1.default.createElement(react_native_1.Pressable, { hitSlop: 10, onPress: onCrop },
                            react_1.default.createElement(react_native_1.Text, { style: styles.text }, okText)))))))));
    return (react_1.default.createElement(react_native_1.Modal, { visible: visible, transparent: true, animationType: "fade", statusBarTranslucent: true, onRequestClose: onClose }, renderContent()));
};
const styles = react_native_1.StyleSheet.create({
    container: {
        backgroundColor: theme_1.Theme.colorTextParagraph,
        width: '100%',
        height: '100%',
    },
    loading: {
        position: 'absolute',
        zIndex: 100,
        left: 0,
        top: 0,
        width: '100%',
        height: '100%',
        justifyContent: 'center',
        alignItems: 'center',
    },
    imageWrap: {
        flex: 1,
        position: 'absolute',
        zIndex: 1,
    },
    gesture: {
        position: 'absolute',
        zIndex: 2,
        left: 0,
        top: 0,
        width: '100%',
        height: '100%',
    },
    rect: {
        position: 'absolute',
        borderWidth: 1,
        borderColor: theme_1.Theme.borderColor,
    },
    footer: {
        position: 'absolute',
        left: 0,
        bottom: 0,
        width: '100%',
        flexDirection: 'row',
        alignItems: 'center',
        justifyContent: 'space-between',
        paddingHorizontal: 20,
        paddingVertical: 20,
    },
    text: {
        fontWeight: theme_1.FONT_WEIGHT.MEDIUM,
        fontSize: 16,
        color: theme_1.Theme.colorTextBaseInverse,
    },
});
exports.default = ImageCropper;
